<?php
define("CHONMAGE_NOT_USE_DB",0);
define("CHONMAGE_USE_DB_PART",1);
define("CHONMAGE_USE_DB_ALL",2);

define("CHONMAGE_KEEP_RATIO",0);
define("CHONMAGE_TRANSFORM",1);
define("CHONMAGE_TRIMMING",2);

class chonmage_config{
    private $config = array();
    private $image_size = array();
    
    
    function __construct(){
        $this->config = parse_ini_file(dirname(__FILE__)."/config.ini");
        $this->image_size = parse_ini_file(dirname(__FILE__)."/image_size.ini");
    }
    
    
    function set_use_sqlite($use_sqlite){
        $this->config["use_sqlite"] = $use_sqlite;
    }
    
    
    function set_sqlite_db_file($path){
        $this->config["sqlite_db_file"] = $path;
    }
    
    
    function configure_mysql($mysql_server,$mysql_user,$mysql_pass,$db_name){
        $this->config["mysql_server"] = $mysql_server;
        $this->config["mysql_user"] = $mysql_user;
        $this->config["mysql_pass"] = $mysql_pass;
        $this->config["db_name"] = $db_name;
    }
    
    
    function setup_db(){
        if($this->config["use_sqlite"]){
            if(!extension_loaded("sqlite3")){
                print "エラー<br>ご利用のPHPにはSQLite3モジュールがインストールされていない、または、SQLite3モジュールが有効化されていません。";
                return FALSE;
            }
            
            $sqlite_obj = new SQLite3(dirname(__FILE__)."/".$this->config["sqlite_db_file"]);
            
            
            $sqlite_r1 = $sqlite_obj->query("SELECT `name` FROM `sqlite_master` WHERE `type` = 'table' AND `name` = 'chonmage_image_blob'");
            if($sqlite_r1 == FALSE){
                print "テーブルの確認に失敗しました。".$sqlite_obj->lastErrorMsg();
                return FALSE;
            }
            
            $q_kekka = $sqlite_r1->fetchArray(SQLITE3_ASSOC);
            if(!$q_kekka){
                $sqlite_r1_2 = $sqlite_obj->query("CREATE TABLE `chonmage_image_blob`(image_id TEXT NOT NULL, size_name TEXT NOT NULL, image_blob BLOB, PRIMARY KEY(image_id,size_name))");
                if($sqlite_r1_2 == FALSE){
                    print "テーブル chonmage_image_blob の作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
            }
            
            
            $sqlite_r2 = $sqlite_obj->query("SELECT `name` FROM `sqlite_master` WHERE `type` = 'table' AND `name` = 'chonmage_image_info'");
            if($sqlite_r2 == FALSE){
                print "テーブルの確認に失敗しました。".$sqlite_obj->lastErrorMsg();
                return FALSE;
            }
            
            $q_kekka = $sqlite_r2->fetchArray(SQLITE3_ASSOC);
            if(!$q_kekka){
                $sqlite_r2_2 = $sqlite_obj->query("CREATE TABLE `chonmage_image_info`(image_id TEXT NOT NULL PRIMARY KEY, original_format TEXT, image_title TEXT, author_id TEXT, description TEXT, created TEXT, last_updated TEXT, views INTEGER NOT NULL DEFAULT 0, score INTEGER NOT NULL DEFAULT 0)");
                if($sqlite_r2_2 == FALSE){
                    print "テーブル chonmage_image_info の作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r2_3 = $sqlite_obj->query("CREATE INDEX `idx_1` ON `chonmage_image_info`(author_id)");
                if($sqlite_r2_3 == FALSE){
                    print "テーブル chonmage_image_info のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r2_4 = $sqlite_obj->query("CREATE INDEX `idx_2` ON `chonmage_image_info`(created)");
                if($sqlite_r2_4 == FALSE){
                    print "テーブル chonmage_image_info のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r2_5 = $sqlite_obj->query("CREATE INDEX `idx_3` ON `chonmage_image_info`(last_updated)");
                if($sqlite_r2_5 == FALSE){
                    print "テーブル chonmage_image_info のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r2_6 = $sqlite_obj->query("CREATE INDEX `idx_4` ON `chonmage_image_info`(views,created)");
                if($sqlite_r2_6 == FALSE){
                    print "テーブル chonmage_image_info のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r2_7 = $sqlite_obj->query("CREATE INDEX `idx_5` ON `chonmage_image_info`(score,created)");
                if($sqlite_r2_7 == FALSE){
                    print "テーブル chonmage_image_info のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
            }
            
            
            $sqlite_r3 = $sqlite_obj->query("SELECT `name` FROM `sqlite_master` WHERE `type` = 'table' AND `name` = 'chonmage_tags'");
            if($sqlite_r3 == FALSE){
                print "テーブルの確認に失敗しました。".$sqlite_obj->lastErrorMsg();
                return FALSE;
            }
            
            $q_kekka = $sqlite_r3->fetchArray(SQLITE3_ASSOC);
            if(!$q_kekka){
                $sqlite_r3_2 = $sqlite_obj->query("CREATE TABLE `chonmage_tags`(image_id TEXT, tag_name TEXT, PRIMARY KEY(tag_name,image_id))");
                if($sqlite_r3_2 == FALSE){
                    print "テーブル chonmage_tags の作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r3_3 = $sqlite_obj->query("CREATE INDEX `idx_6` ON `chonmage_tags`(image_id)");
                if($sqlite_r3_3 == FALSE){
                    print "テーブル chonmage_tags のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
            }
            
            
            $sqlite_r4 = $sqlite_obj->query("SELECT `name` FROM `sqlite_master` WHERE `type` = 'table' AND `name` = 'chonmage_comments'");
            if($sqlite_r4 == FALSE){
                print "テーブルの確認に失敗しました。".$sqlite_obj->lastErrorMsg();
                return FALSE;
            }
            
            $q_kekka = $sqlite_r4->fetchArray(SQLITE3_ASSOC);
            if(!$q_kekka){
                $sqlite_r4_2 = $sqlite_obj->query("CREATE TABLE `chonmage_comments`(image_id TEXT, author_id TEXT, content TEXT, created TEXT, last_updated TEXT)");
                if($sqlite_r4_2 == FALSE){
                    print "テーブル chonmage_comments の作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r4_3 = $sqlite_obj->query("CREATE INDEX `idx_7` ON `chonmage_comments`(author_id)");
                if($sqlite_r4_3 == FALSE){
                    print "テーブル chonmage_comments のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
                
                $sqlite_r4_4 = $sqlite_obj->query("CREATE INDEX `idx_8` ON `chonmage_comments`(image_id,created)");
                if($sqlite_r4_4 == FALSE){
                    print "テーブル chonmage_comments のインデックスの作成に失敗しました。".$sqlite_obj->lastErrorMsg();
                    return FALSE;
                }
            }
            
            $sqlite_obj->close();
        }else{
            $mysql_obj = new mysqli($this->config["mysql_server"],$this->config["mysql_user"],$this->config["mysql_pass"]);
            if ($mysql_obj->connect_errno){
                print "データベース接続に失敗しました。".$mysql_obj->connect_error;
                return FALSE;
            }
            
            $mysql_obj->set_charset("utf8");
            
            $mysql_r1 = $mysql_obj->query("SHOW DATABASES LIKE '".$this->config["db_name"]."'");
            if ($mysql_r1 == FALSE){
                print "データベースの確認に失敗しました。".$mysql_obj->error;
                return FALSE;
            }
            
            $q_kekka = $mysql_r1->fetch_assoc();
            if(!$q_kekka){
                $mysql_r1_2 = $mysql_obj->query("CREATE DATABASE `".$this->config["db_name"]."` default character set utf8");
                if ($mysql_r1_2 == FALSE){
                    print "データベースの作成に失敗しました。".$mysql_obj->error."<br>データベース作成権限をご確認ください。";
                    return FALSE;
                }
            }
            
            $mysql_obj->select_db($this->config["db_name"]);
            
            $mysql_r2 = $mysql_obj->query("SHOW TABLES FROM `".$this->config["db_name"]."` LIKE 'chonmage_image_blob'");
            if ($mysql_r2 == FALSE){
                print "テーブルの確認に失敗しました。".$mysql_obj->error;
                return FALSE;
            }
            
            if($mysql_r2->fetch_assoc() == NULL){
                $mysql_r2_2 = $mysql_obj->query("CREATE TABLE `chonmage_image_blob`(image_id CHAR(26) NOT NULL, size_name CHAR(63) NOT NULL, image_blob MEDIUMBLOB, PRIMARY KEY(image_id,size_name))");
                if ($mysql_r2_2 == FALSE){
                    print "テーブル chonmage_image_blob の作成に失敗しました。".$mysql_obj->error;
                    return FALSE;
                }
            }
            
            $mysql_r3 = $mysql_obj->query("SHOW TABLES FROM `".$this->config["db_name"]."` LIKE 'chonmage_image_info'");
            if ($mysql_r3 == FALSE){
                print "テーブルの確認に失敗しました。".$mysql_obj->error;
                return FALSE;
            }
            
            if($mysql_r3->fetch_assoc() == NULL){
                $mysql_r3_2 = $mysql_obj->query("CREATE TABLE `chonmage_image_info`(image_id CHAR(26) NOT NULL PRIMARY KEY, original_format CHAR(4), image_title CHAR(127), author_id CHAR(64), description TEXT, created DATETIME, last_updated DATETIME, views INT NOT NULL DEFAULT 0, score MEDIUMINT NOT NULL DEFAULT 0, INDEX idx_1(author_id), INDEX idx_2(created), INDEX idx_3(last_updated), INDEX idx_4(views,created), INDEX idx_5(score,created))");
                if ($mysql_r3_2 == FALSE){
                    print "テーブル chonmage_image_info の作成に失敗しました。".$mysql_obj->error;
                    return FALSE;
                }
            }
            
            $mysql_r4 = $mysql_obj->query("SHOW TABLES FROM `".$this->config["db_name"]."` LIKE 'chonmage_tags'");
            if ($mysql_r4 == FALSE){
                print "テーブルの確認に失敗しました。".$mysql_obj->error;
            }
            
            if($mysql_r4->fetch_assoc() == NULL){
                $mysql_r4_2 = $mysql_obj->query("CREATE TABLE `chonmage_tags`(image_id CHAR(26), tag_name CHAR(63), PRIMARY KEY(tag_name,image_id), INDEX idx_6 (image_id))");
                if ($mysql_r4_2 == FALSE){
                    print "テーブル chonmage_tags の作成に失敗しました。".$mysql_obj->error;
                    return FALSE;
                }
            }
            
            $mysql_r5 = $mysql_obj->query("SHOW TABLES FROM `".$this->config["db_name"]."` LIKE 'chonmage_comments'");
            if ($mysql_r5 == FALSE){
                print "テーブルの確認に失敗しました。".$mysql_obj->error;
            }
            
            if($mysql_r5->fetch_assoc() == NULL){
                $mysql_r5_2 = $mysql_obj->query("CREATE TABLE `chonmage_comments`(image_id CHAR(26), author_id CHAR(64), content TEXT, created DATETIME, last_updated DATETIME, INDEX idx_7 (author_id), INDEX idx_8 (image_id,created))");
                if ($mysql_r5_2 == FALSE){
                    print "テーブル chonmage_comments の作成に失敗しました。".$mysql_obj->error;
                    return FALSE;
                }
            }
            
            $mysql_obj->close();
            return TRUE;
        }
    }
    
    
    function set_use_db($use_db){
        $this->config["use_db"] = $use_db;
    }
    
    
    function set_save_original($save_original){
        $this->config["save_original"] = $save_original;
    }
    
    
    function set_image_dir($image_dir){
        if(file_exists(dirname(__FILE__)."/".$image_dir)){
            $this->config["image_dir"] = $image_dir;
            return TRUE;
        }else{
            return FALSE;
        }
    }
    
    
    function add_image_size($size_name,$width,$height,$file_format = "jpeg",$background = "#ffffff",$expand = FALSE,$scaling_method = CHONMAGE_KEEP_RATIO,$compression_quality = 75){
        if($size_name === "original" || strlen($size_name) > 63){
            return FALSE;
        }
        
        $image_size_key = array_search($size_name,$this->image_size["size_name"]);
        
        if($image_size_key === FALSE || $image_size_key === NULL){
            $this->image_size["size_name"][] = $size_name;
            $this->image_size["width"][] = $width;
            $this->image_size["height"][] = $height;
            $this->image_size["background"][] = $background;
            $this->image_size["file_format"][] = $file_format;
            $this->image_size["expand"][] = $expand;
            $this->image_size["scaling_method"][] = $scaling_method;
            $this->image_size["compression_quality"][] = $compression_quality;
            
            return TRUE;
        }else{
            return FALSE;
        }
    }
    
    
    function delete_image_size($size_name){
        $image_size_key = array_search($size_name,$this->image_size["size_name"]);
        
        if($image_size_key === FALSE){
            return FALSE;
        }else{
            unset($this->image_size["size_name"][$image_size_key]);
            $this->image_size["size_name"] = array_values($this->image_size["size_name"]);
            unset($this->image_size["width"][$image_size_key]);
            $this->image_size["width"] = array_values($this->image_size["width"]);
            unset($this->image_size["height"][$image_size_key]);
            $this->image_size["height"] = array_values($this->image_size["height"]);
            unset($this->image_size["background"][$image_size_key]);
            $this->image_size["background"] = array_values($this->image_size["background"]);
            unset($this->image_size["file_format"][$image_size_key]);
            $this->image_size["file_format"] = array_values($this->image_size["file_format"]);
            unset($this->image_size["expand"][$image_size_key]);
            $this->image_size["expand"] = array_values($this->image_size["expand"]);
            unset($this->image_size["scaling_method"][$image_size_key]);
            $this->image_size["scaling_method"] = array_values($this->image_size["scaling_method"]);
            unset($this->image_size["compression_quality"][$image_size_key]);
            $this->image_size["compression_quality"] = array_values($this->image_size["compression_quality"]);
            
            return TRUE;
        }
    }
    
    
    function save(){
        $file_h = fopen(dirname(__FILE__)."/config.ini","w");
        
        if($this->config["use_sqlite"]){
            fwrite($file_h,"use_sqlite=true\n");
        }else{
            fwrite($file_h,"use_sqlite=false\n");
        }
        fwrite($file_h,"sqlite_db_file=\"".$this->config["sqlite_db_file"]."\"\n");
        fwrite($file_h,"\n");
        fwrite($file_h,"mysql_server=\"".$this->config["mysql_server"]."\"\n");
        fwrite($file_h,"mysql_user=\"".$this->config["mysql_user"]."\"\n");
        fwrite($file_h,"mysql_pass=\"".$this->config["mysql_pass"]."\"\n");
        fwrite($file_h,"db_name=\"".$this->config["db_name"]."\"\n");
        fwrite($file_h,"\n");
        fwrite($file_h,"use_db=".$this->config["use_db"]."\n");
        if($this->config["save_original"]){
            fwrite($file_h,"save_original=true\n");
        }else{
            fwrite($file_h,"save_original=false\n");
        }
        fwrite($file_h,"\n");
        fwrite($file_h,"image_dir=\"".$this->config["image_dir"]."\"\n");
        
        fclose($file_h);
        
        
        $file_h = fopen(dirname(__FILE__)."/image_size.ini","w");
        
        for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
            fwrite($file_h,"size_name[".$cnt."]=\"".$this->image_size["size_name"][$cnt]."\"\n");
            fwrite($file_h,"width[".$cnt."]=".$this->image_size["width"][$cnt]."\n");
            fwrite($file_h,"height[".$cnt."]=".$this->image_size["height"][$cnt]."\n");
            fwrite($file_h,"background[".$cnt."]=\"".$this->image_size["background"][$cnt]."\"\n");
            fwrite($file_h,"file_format[".$cnt."]=\"".$this->image_size["file_format"][$cnt]."\"\n");
            if($this->image_size["expand"][$cnt]){
                fwrite($file_h,"expand[".$cnt."]=true\n");
            }else{
                fwrite($file_h,"expand[".$cnt."]=false\n");
            }
            fwrite($file_h,"scaling_method[".$cnt."]=".$this->image_size["scaling_method"][$cnt]."\n");
            fwrite($file_h,"compression_quality[".$cnt."]=".$this->image_size["compression_quality"][$cnt]."\n");
            fwrite($file_h,"\n");
        }
        
        fclose($file_h);
    }
}

function chonmage_info(){
    print "<img src='data:image/png;base64,".base64_encode(file_get_contents(dirname(__FILE__)."/chonmage_logo.png"))."' alt='Chonmage'><br>\n";
    print "<span style='font-size: xx-large;'>Chonm-age 1.3</span><br>\n";
    print "<b>無権利創作機構 きのくにソフトウェア</b> <a href='http://soft.noright.org/'>http://soft.noright.org/</a><br>\n";
    print "このソフトウェアは、無権利創作ライセンス(<a href='http://www.noright.org/license/'>http://www.noright.org/license/</a>)に従って著作権放棄されています。";
}
