<?php
class chonmage{
    private $config = array();
    private $image_size = array();
    private $db_obj = array();
    private $escape_list = array("\0","..");
    
    private $image_blob = array();
    private $format;
    
    function __construct() {
        $this->config = parse_ini_file(dirname(__FILE__)."/config.ini");
        $this->image_size = parse_ini_file(dirname(__FILE__)."/image_size.ini");
        
        if($this->config["use_db"] >= 1){
            if($this->config["use_sqlite"]){
                $this->db_obj = new SQLite3(dirname(__FILE__)."/".$this->config["sqlite_db_file"]);
            }else{
                $this->db_obj = new mysqli($this->config["mysql_server"],$this->config["mysql_user"],$this->config["mysql_pass"],$this->config["db_name"]);
                if($this->db_obj->connect_errno){
                    print "MySQL接続に失敗しました。".$this->db_obj->connect_error;
                    return FALSE;
                }
                
                $this->db_obj->set_charset("utf8");
            }
        }
    }
    
    
    private function escape_image_id($image_id){
        return str_replace("..","",preg_replace("/[^0-9\/\.]/","",$image_id));
    }
    
    
    private function make_blob($image_file_name){
        if(extension_loaded("imagick")){
            $img = new Imagick();
            if(!$img->readImage($image_file_name)){
                print "画像を読み込めませんでした。";
                return FALSE;
            }
            
            $this->format = strtolower($img->getImageFormat());
            
            if($this->format === "jpeg"){
                $kaiten = $img->getImageOrientation();
                
                switch($kaiten){
                    case imagick::ORIENTATION_RIGHTTOP:
                        $img->rotateImage(new ImagickPixel(),90);
                        break;
                    case imagick::ORIENTATION_BOTTOMRIGHT:
                        $img->rotateImage(new ImagickPixel(),180);
                        break;
                    case imagick::ORIENTATION_LEFTBOTTOM:
                        $img->rotateImage(new ImagickPixel(),270);
                        break;
                }
                
                $img->setImageOrientation(imagick::ORIENTATION_TOPLEFT);
            }
            
            for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                $img_after = clone $img;
                
                if($img_after->getImageWidth() > $this->image_size["width"][$cnt] || $img_after->getImageHeight() > $this->image_size["height"][$cnt] || $this->image_size["expand"][$cnt]){
                    switch($this->image_size["scaling_method"][$cnt]){
                        case 0:
                            if($this->image_size["width"][$cnt] > 300){
                                $img_after->resizeImage($this->image_size["width"][$cnt],$this->image_size["height"][$cnt],imagick::FILTER_LANCZOS,1,TRUE);
                            }else{
                                $img_after->thumbnailImage($this->image_size["width"][$cnt],$this->image_size["height"][$cnt],TRUE);
                            }
                            break;
                        case 1:
                            if($this->image_size["width"][$cnt] > 300){
                                $img_after->resizeImage($this->image_size["width"][$cnt],$this->image_size["height"][$cnt],imagick::FILTER_LANCZOS,1,FALSE);
                            }else{
                                $img_after->thumbnailImage($this->image_size["width"][$cnt],$this->image_size["height"][$cnt],FALSE);
                            }
                            break;
                        case 2:
                            $x_hiritsu = $img_after->getImageWidth() / $this->image_size["width"][$cnt];
                            $y_hiritsu = $img_after->getImageHeight() / $this->image_size["height"][$cnt];
                            
                            if($x_hiritsu >= $y_hiritsu){
                                $img_after->cropThumbnailImage($this->image_size["width"][$cnt],$this->image_size["height"][$cnt]);
                            }else{
                                $img_after->thumbnailImage($this->image_size["width"][$cnt],0);
                                $img_after->cropImage($this->image_size["width"][$cnt],$this->image_size["height"][$cnt],0,0);
                                $img_after->setImagePage($this->image_size["width"][$cnt],$this->image_size["height"][$cnt],0,0);
                            }
                            break;
                    }
                }
                
                if($this->image_size["file_format"][$cnt] === "jpeg"){
                    $img_after->setImageCompressionQuality($this->image_size["compression_quality"][$cnt]);
                    $img_after->setImageBackgroundColor($this->image_size["background"][$cnt]);
                    $img_after = $img_after->flattenImages();
                }
                $img_after->setImageFormat($this->image_size["file_format"][$cnt]);
                $this->image_blob[$cnt] = $img_after->getImageBlob();
                $img_after->clear();
            }
            
        }else{
            
            switch(exif_imagetype($image_file_name)){
                case IMAGETYPE_PNG:
                    $this->format = "png";
                    $img = imagecreatefrompng($image_file_name);
                    break;
                case IMAGETYPE_JPEG:
                    $this->format = "jpeg";
                    $img = imagecreatefromjpeg($image_file_name);
                    $exif = exif_read_data($image_file_name);
                    switch($exif["Orientation"]){
                        case 3:
                            $img = imagerotate($img,180,0);
                            break;
                        case 6:
                            $img = imagerotate($img,270,0);
                            break;
                        case 8:
                            $img = imagerotate($img,90,0);
                            break;
                    }
                    break;
                case IMAGETYPE_GIF:
                    $this->format = "gif";
                    $img = imagecreatefromgif($image_file_name);
                    break;
                default:
                    print "指定された画像ファイルは対応していない形式です。";
                    return FALSE;
            }
            
            for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                $img_x = imagesx($img);
                $img_y = imagesy($img);
                if($img_x > $this->image_size["width"][$cnt] || $img_y > $this->image_size["height"][$cnt] || $this->image_size["expand"][$cnt]){
                    switch($this->image_size["scaling_method"][$cnt]){
                        case 0:
                            $x_hiritsu = $img_x / $this->image_size["width"][$cnt];
                            $y_hiritsu = $img_y / $this->image_size["height"][$cnt];
                            
                            if($x_hiritsu >= $y_hiritsu){
                                $x_after = $this->image_size["width"][$cnt];
                                $y_after = round($img_y / $x_hiritsu);
                            }else{
                                $x_after = round($img_x / $y_hiritsu);
                                $y_after = $this->image_size["height"][$cnt];
                            }
                            
                            $src_x = $img_x;
                            $src_y = $img_y;
                            $src_offset_x = 0;
                            
                            break;
                        case 1:
                            $x_after = $this->image_size["width"][$cnt];
                            $y_after = $this->image_size["height"][$cnt];
                            $src_x = $img_x;
                            $src_y = $img_y;
                            $src_offset_x = 0;
                            
                            break;
                        case 2:
                            $x_after = $this->image_size["width"][$cnt];
                            $y_after = $this->image_size["height"][$cnt];
                            
                            $x_hiritsu = $img_x / $this->image_size["width"][$cnt];
                            $y_hiritsu = $img_y / $this->image_size["height"][$cnt];
                            
                            if($x_hiritsu >= $y_hiritsu){
                                $src_x = round($this->image_size["width"][$cnt] * $y_hiritsu);
                                $src_y = $img_y;
                                
                                $src_offset_x = round(($img_x - $src_x) / 2);
                            }else{
                                $src_x = $img_x;
                                $src_y = round($this->image_size["height"][$cnt] * $x_hiritsu);
                                $src_offset_x = 0;
                            }
                            
                            break;
                    }
                }else{
                    $x_after = $img_x;
                    $y_after = $img_y;
                    
                    $src_x = $img_x;
                    $src_y = $img_y;
                    $src_offset_x = 0;
                }
                
                $img_after = imagecreatetruecolor($x_after,$y_after);
                
                switch($this->image_size["file_format"][$cnt]){
                    case "jpeg":
                    case "gif":
                        imagefilledrectangle($img_after,0,0,$x_after,$y_after,imagecolorallocate($img_after,hexdec(substr($this->image_size["background"][$cnt],1,2)),hexdec(substr($this->image_size["background"][$cnt],3,2)),hexdec(substr($this->image_size["background"][$cnt],5,2))));
                        break;
                    case "png":
                        imagealphablending($img_after,FALSE);
                        imagesavealpha($img_after,TRUE);
                        break;
                }
                
                imagecopyresampled($img_after,$img,0,0,$src_offset_x,0,$x_after,$y_after,$src_x,$src_y);
                
                ob_start();
                switch($this->image_size["file_format"][$cnt]){
                    case "jpeg":
                        imagejpeg($img_after,NULL,$this->image_size["compression_quality"][$cnt]);
                        break;
                    case "png":
                        imagepng($img_after);
                        break;
                    case "gif":
                        imagegif($img_after);
                        break;
                }
                
                $this->image_blob[$cnt] = ob_get_clean();
            }
        }
    }
    
    
    private function adjust_tags($tags){
        $tag_henkan_before = str_replace("　"," ",$tags);
        
        /*不必要な半角スペースの削除*/
        while(1){
            $tag_henkan_after = str_replace("  "," ",$tag_henkan_before);/*2連続のスペースを1連続に変換*/
            if($tag_henkan_after === $tag_henkan_before){
                break;
            }
            $tag_henkan_before = $tag_henkan_after;
        }
        
        /*先頭のスペースを削除*/
        $kekka_pos = strpos($tag_henkan_after," ",0);
        if($kekka_pos === 0){
            $tag_henkan_after = substr($tag_henkan_after,1);
        }
        
        /*末尾のスペースを削除*/
        $tag_nagasa = strlen($tag_henkan_after);
        if($tag_nagasa != 0){
            $kekka_pos = strpos($tag_henkan_after," ",$tag_nagasa - 1);
            if($kekka_pos !== FALSE){
                $tag_henkan_after = substr($tag_henkan_after,0,$tag_nagasa - 1);
            }
        }
        
        $offset = -1;
        for($cnt = 0; $offset !== FALSE; $cnt++){
            if($cnt >= 10){
                $tag_henkan_after = substr($tag_henkan_after,0,$offset);
                break;
            }
            $offset = strpos($tag_henkan_after," ",$offset + 1);
        }
        
        return $tag_henkan_after;
    }
    
    
    function add_image($up_file,$author_id,$title,$description,$tags,$save_original = NULL){
        $image_id = date("Y/m/d/H.i.s").".".substr(microtime(),2,6);
        
        if($save_original === NULL){
            $save_original = $this->config["save_original"];
        }
        
        $this->make_blob($up_file);
        
        $tag_henkan_after = $this->adjust_tags($tags);
        
        $ima = date("Y-m-d H:i:s");
        
        if($this->config["use_db"] <= 1){
            $base_dir = dirname(__FILE__)."/".$this->config["image_dir"]."/";
            
            $path_y = date("Y")."/";
            if(file_exists($base_dir.$path_y) == FALSE){
                mkdir($base_dir.$path_y);
            }
            
            $path_ym = $path_y.date("m")."/";
            if(file_exists($base_dir.$path_ym) == FALSE){
                mkdir($base_dir.$path_ym);
            }
            
            $path_ymd = $path_ym.date("d")."/";
            $ymd_dir = $base_dir.$path_ymd;
            if(file_exists($ymd_dir) == FALSE){
                mkdir($ymd_dir);
            }
            
            for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                file_put_contents($base_dir.$image_id."_".$this->image_size["size_name"][$cnt].".".$this->image_size["file_format"][$cnt],$this->image_blob[$cnt]);
            }
            
            if($save_original){
                copy($up_file,$base_dir.$image_id.".".$this->format);
            }
            
            $ini_h = fopen($base_dir.$image_id.".ini","w");
            fwrite($ini_h,"image_id=\"".$image_id."\"\n");
            fwrite($ini_h,"author_id=\"".addslashes($author_id)."\"\n");
            fwrite($ini_h,"original_format=\"".$this->format."\"\n");
            fwrite($ini_h,"image_title=\"".addslashes($title)."\"\n");
            fwrite($ini_h,"description=\"".addslashes($description)."\"\n");
            fwrite($ini_h,"created=\"".$ima."\"\n");
            fwrite($ini_h,"last_updated=\"".$ima."\"\n");
            fwrite($ini_h,"views=0\n");
            fwrite($ini_h,"score=0\n");
            fwrite($ini_h,"tags=\"".addslashes($tag_henkan_after)."\"\n");
            fclose($ini_h);
            
            $list_h = fopen($ymd_dir."image_list.ini","a");
            fwrite($list_h,$image_id."\n");
            fclose($list_h);
            
            $rireki_path = dirname(__FILE__)."/".$this->config["image_dir"]."/recent_images.ini";
            if(file_exists($rireki_path)){
                $rireki = file_get_contents($rireki_path);
            }else{
                $rireki = "";
            }
            
            file_put_contents($rireki_path,$image_id."\n".substr($rireki,0,1323));/*  1323 = 27 * 49  */
        }
        
        if($this->config["use_db"] >= 1){
            if($this->config["use_db"] == 2){
                if($this->config["use_sqlite"]){
                    $r1 = $this->db_obj->query("INSERT INTO `chonmage_image_info`(`image_id`,`original_format`,`image_title`,`author_id`,`created`,`last_updated`,`description`) VALUES ('".$image_id."','".$this->db_obj->escapeString($this->format)."','".$this->db_obj->escapeString($title)."','".$this->db_obj->escapeString($author_id)."','".$ima."','".$ima."','".$this->db_obj->escapeString($description)."')");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                        return FALSE;
                    }
                    
                    if($save_original){
                        $stmt_2_1 = $this->db_obj->prepare("INSERT INTO `chonmage_image_blob`(`image_id`,`size_name`,`image_blob`) VALUES ('".$image_id."','original',:image_blob)");
                        $stmt_2_1->bindValue(":image_blob",file_get_contents($up_file),SQLITE3_BLOB);
                        $r2_1 = $stmt_2_1->execute();
                        if($r2_1 == FALSE){
                            print "SQLクエリ2-1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                            return FALSE;
                        }
                    }
                    
                    for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                        $stmt_2_2 = $this->db_obj->prepare("INSERT INTO `chonmage_image_blob`(`image_id`,`size_name`,`image_blob`) VALUES ('".$image_id."','".$this->db_obj->escapeString($this->image_size["size_name"][$cnt])."',:image_blob)");
                        $stmt_2_2->bindValue(":image_blob",$this->image_blob[$cnt],SQLITE3_BLOB);
                        $r2_2 = $stmt_2_2->execute();
                        if($r2_2 == FALSE){
                            print "SQLクエリ2-2の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                            return FALSE;
                        }
                    }
                }else{
                    $r1 = $this->db_obj->query("INSERT INTO `chonmage_image_info`(`image_id`,`original_format`,`image_title`,`author_id`,`created`,`last_updated`,`description`) VALUES ('".$image_id."','".$this->db_obj->real_escape_string($this->format)."','".$this->db_obj->real_escape_string($title)."','".$this->db_obj->real_escape_string($author_id)."','".$ima."','".$ima."','".$this->db_obj->real_escape_string($description)."')");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                        return FALSE;
                    }
                    
                    if($save_original){
                        $r2_1 = $this->db_obj->query("INSERT INTO `chonmage_image_blob`(`image_id`,`size_name`,`image_blob`) VALUES ('".$image_id."','original','".$this->db_obj->real_escape_string(file_get_contents($up_file))."')");
                        if($r2_1 == FALSE){
                            print "SQLクエリ2-1の実行に失敗しました。".$this->db_obj->error;
                            return FALSE;
                        }
                    }
                    
                    for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                        $r2_2 = $this->db_obj->query("INSERT INTO `chonmage_image_blob`(`image_id`,`size_name`,`image_blob`) VALUES ('".$image_id."','".$this->db_obj->real_escape_string($this->image_size["size_name"][$cnt])."','".$this->db_obj->real_escape_string($this->image_blob[$cnt])."')");
                        if($r2_2 == FALSE){
                            print "SQLクエリ2-2の実行に失敗しました。".$this->db_obj->error;
                            return FALSE;
                        }
                    }
                }
            }else{
                if($this->config["use_sqlite"]){
                    $r1 = $this->db_obj->query("INSERT INTO `chonmage_image_info`(`image_id`,`original_format`,`image_title`,`author_id`,`created`,`last_updated`) VALUES ('".$image_id."','".$this->db_obj->escapeString($this->format)."','".$this->db_obj->escapeString($title)."','".$this->db_obj->escapeString($author_id)."','".$ima."','".$ima."')");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                        return FALSE;
                    }
                }else{
                    $r1 = $this->db_obj->query("INSERT INTO `chonmage_image_info`(`image_id`,`original_format`,`image_title`,`author_id`,`created`,`last_updated`) VALUES ('".$image_id."','".$this->db_obj->real_escape_string($this->format)."','".$this->db_obj->real_escape_string($title)."','".$this->db_obj->real_escape_string($author_id)."','".$ima."','".$ima."')");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                        return FALSE;
                    }
                }
            }
            
            if(strlen($tag_henkan_after) >= 1){
                $tag = explode(" ",$tag_henkan_after);
                $tag = array_unique($tag);
                $tag_kosu = count($tag);
                
                for($tag_c = 0; $tag_c < $tag_kosu; $tag_c++){
                    if($this->config["use_sqlite"]){
                        $r3 = $this->db_obj->query("INSERT INTO `chonmage_tags`(`image_id`,`tag_name`) VALUES ('".$image_id."','".$tag[$tag_c]."')");
                        if($r3 == FALSE){
                            print "SQLクエリ3の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                            return FALSE;
                        }
                    }else{
                        $r3 = $this->db_obj->query("INSERT INTO `chonmage_tags`(`image_id`,`tag_name`) VALUES ('".$image_id."','".$tag[$tag_c]."')");
                        if($r3 == FALSE){
                            print "SQLクエリ3の実行に失敗しました。".$this->db_obj->error;
                            return FALSE;
                        }
                    }
                }
            }
        }
        
        return $image_id;
    }
    
    
    function replace_image($image_id,$up_file,$save_original = NULL){
        $image_id = $this->escape_image_id($image_id);
        
        if($save_original === NULL){
            $save_original = $this->config["save_original"];
        }
        
        $this->make_blob($up_file);
        
        $ima = date("Y-m-d H:i:s");
        
        if($this->config["use_db"] <= 1){
            $base_dir = dirname(__FILE__)."/".$this->config["image_dir"]."/";
            
            for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                file_put_contents($base_dir.$image_id."_".$this->image_size["size_name"][$cnt].".".$this->image_size["file_format"][$cnt],$this->image_blob[$cnt]);
            }
            
            if($save_original){
                copy($up_file,$base_dir.$image_id.".".$this->format);
            }
            
            $ini_data = parse_ini_file($base_dir.$image_id.".ini");
            
            $ini_h = fopen($base_dir.$image_id.".ini","w");
            fwrite($ini_h,"image_id=\"".$image_id."\"\n");
            fwrite($ini_h,"author_id=\"".addslashes($ini_data["author_id"])."\"\n");
            fwrite($ini_h,"original_format=\"".$this->format."\"\n");
            fwrite($ini_h,"image_title=\"".addslashes($ini_data["image_title"])."\"\n");
            fwrite($ini_h,"description=\"".addslashes($ini_data["description"])."\"\n");
            fwrite($ini_h,"created=\"".$ini_data["created"]."\"\n");
            fwrite($ini_h,"last_updated=\"".$ima."\"\n");
            fwrite($ini_h,"views=".$ini_data["views"]."\n");
            fwrite($ini_h,"score=".$ini_data["score"]."\n");
            fwrite($ini_h,"tags=\"".addslashes($ini_data["tags"])."\"\n");
            fclose($ini_h);
        }
        
        if($this->config["use_db"] >= 1){
            if($this->config["use_db"] == 2){
                if($save_original){
                    if($this->config["use_sqlite"]){
                        $r1 = $this->db_obj->query("SELECT `image_blob` FROM `chonmage_image_blob` WHERE image_id = '".$image_id."' AND size_name = 'original'");
                        if($r1 == FALSE){
                            print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                            return FALSE;
                        }
                        $kekka = $r1->fetchArray(SQLITE3_ASSOC);
                        if($kekka === NULL){
                            $stmt_2_1 = $this->db_obj->prepare("INSERT INTO `chonmage_image_blob`(`image_id`,`size_name`,`image_blob`) VALUES ('".$image_id."','original',:image_blob)");
                            $stmt_2_1->bindValue(":image_blob",file_get_contents($up_file),SQLITE3_BLOB);
                            $r2_1 = $stmt_2_1->execute();
                            if($r2_1 == FALSE){
                                print "SQLクエリ2-1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                                return FALSE;
                            }
                        }else{
                            $stmt_2_1 = $this->db_obj->prepare("UPDATE `chonmage_image_blob` SET `image_blob` = :image_blob WHERE `image_id` = '".$image_id."' AND `size_name` = 'original'");
                            $stmt_2_1->bindValue(":image_blob",file_get_contents($up_file),SQLITE3_BLOB);
                            $r2_1 = $stmt_2_1->execute();
                            if($r2_1 == FALSE){
                                print "SQLクエリ2-1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                                return FALSE;
                            }
                        }
                    }else{
                        $r1 = $this->db_obj->query("SELECT `image_blob` FROM `chonmage_image_blob` WHERE image_id = '".$image_id."' AND size_name = 'original'");
                        if($r1 == FALSE){
                            print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                            return FALSE;
                        }
                        $kekka = $r1->fetch_assoc();
                        if($kekka === NULL){
                            $r2_1 = $this->db_obj->query("INSERT INTO `chonmage_image_blob`(`image_id`,`size_name`,`image_blob`) VALUES ('".$image_id."','original','".$this->db_obj->real_escape_string(file_get_contents($up_file))."')");
                            if($r2_1 == FALSE){
                                print "SQLクエリ2-1の実行に失敗しました。".$this->db_obj->error;
                                return FALSE;
                            }
                        }else{
                            $r2_1 = $this->db_obj->query("UPDATE `chonmage_image_blob` SET `image_blob` = '".$this->db_obj->real_escape_string(file_get_contents($up_file))."' WHERE `image_id` = '".$image_id."' AND `size_name` = 'original'");
                            if($r2_1 == FALSE){
                                print "SQLクエリ2-1の実行に失敗しました。".$this->db_obj->error;
                                return FALSE;
                            }
                        }
                    }
                }else{
                    if($this->config["use_sqlite"]){
                        $r2_3 = $this->db_obj->query("DELETE FROM `chonmage_image_blob` WHERE `image_id` = '".$image_id."' AND `size_name` = 'original'");
                        if($r2_3 == FALSE){
                            print "SQLクエリ2-3の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                            return FALSE;
                        }
                    }else{
                        $r2_3 = $this->db_obj->query("DELETE FROM `chonmage_image_blob` WHERE `image_id` = '".$image_id."' AND `size_name` = 'original'");
                        if($r2_3 == FALSE){
                            print "SQLクエリ2-3の実行に失敗しました。".$this->db_obj->error;
                            return FALSE;
                        }
                    }
                }
            
                for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                    if($this->config["use_sqlite"]){
                        $stmt_2_2 = $this->db_obj->prepare("UPDATE `chonmage_image_blob` SET `image_blob` = :image_blob WHERE `image_id` = '".$image_id."' AND `size_name` = '".$this->db_obj->escapeString($this->image_size["size_name"][$cnt])."'");
                        $stmt_2_2->bindValue(":image_blob",$this->image_blob[$cnt],SQLITE3_BLOB);
                        $r2_2 = $stmt_2_2->execute();
                        if($r2_2 == FALSE){
                            print "SQLクエリ2-2の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                            return FALSE;
                        }
                    }else{
                        $r2_2 = $this->db_obj->query("UPDATE `chonmage_image_blob` SET `image_blob` = '".$this->db_obj->real_escape_string($this->image_blob[$cnt])."' WHERE `image_id` = '".$image_id."' AND `size_name` = '".$this->db_obj->real_escape_string($this->image_size["size_name"][$cnt])."'");
                        if($r2_2 == FALSE){
                            print "SQLクエリ2-2の実行に失敗しました。".$this->db_obj->error;
                            return FALSE;
                        }
                    }
                }
            }
            
            if($this->config["use_sqlite"]){
                $r3 = $this->db_obj->query("UPDATE `chonmage_image_info` SET `original_format` = '".$this->db_obj->escapeString($this->format)."',`last_updated` = '".$ima."' WHERE `image_id` = '".$image_id."'");
                if($r3 == FALSE){
                    print "SQLクエリ3の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
            }else{
                $r3 = $this->db_obj->query("UPDATE `chonmage_image_info` SET `original_format` = '".$this->db_obj->real_escape_string($this->format)."',`last_updated` = '".$ima."' WHERE `image_id` = '".$image_id."'");
                if($r3 == FALSE){
                    print "SQLクエリ3の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
            }
        }
        
        return TRUE;
    }
    
    
    function delete_image($image_id){
        $image_id = $this->escape_image_id($image_id);
        
        if($this->config["use_db"] <= 1){
            $base_dir = dirname(__FILE__)."/".$this->config["image_dir"]."/";
            
            if(!file_exists($base_dir.$image_id.".ini")){
                return FALSE;
            }
            $ini_data = parse_ini_file($base_dir.$image_id.".ini");
            
            unlink($base_dir.$image_id.".ini");
            for($cnt = 0; isset($this->image_size["size_name"][$cnt]); $cnt++){
                unlink($base_dir.$image_id."_".$this->image_size["size_name"][$cnt].".".$this->image_size["file_format"][$cnt]);
            }
            if(file_exists($base_dir.$image_id.".".$ini_data["original_format"])){
                unlink($base_dir.$image_id.".".$ini_data["original_format"]);
            }
            
            $list_path = $base_dir.dirname($image_id)."/image_list.ini";
            file_put_contents($list_path,str_replace($image_id."\n","",file_get_contents($list_path)));
            
            $rireki_path = $base_dir."recent_images.ini";
            file_put_contents($rireki_path,str_replace($image_id."\n","",file_get_contents($rireki_path)));
        }
        
        if($this->config["use_db"] >= 1){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("DELETE FROM `chonmage_image_info` WHERE `image_id` = '".$image_id."'");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                    
                $r2 = $this->db_obj->query("DELETE FROM `chonmage_image_blob` WHERE `image_id` = '".$image_id."'");
                if($r2 == FALSE){
                    print "SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                    
                $r3 = $this->db_obj->query("DELETE FROM `chonmage_tags` WHERE `image_id` = '".$image_id."'");
                if($r3 == FALSE){
                    print "SQLクエリ3の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
            }else{
                $r1 = $this->db_obj->query("DELETE FROM `chonmage_image_info` WHERE `image_id` = '".$image_id."'");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                    
                $r2 = $this->db_obj->query("DELETE FROM `chonmage_image_blob` WHERE `image_id` = '".$image_id."'");
                if($r2 == FALSE){
                    print "SQLクエリ2の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                    
                $r3 = $this->db_obj->query("DELETE FROM `chonmage_tags` WHERE `image_id` = '".$image_id."'");
                if($r3 == FALSE){
                    print "SQLクエリ3の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
            }
        }
        
        return TRUE;
    }
    
    
    function get_image_info($image_id){
        $image_id = $this->escape_image_id($image_id);
        
        if($this->config["use_db"] == 2){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT `original_format`,`image_title`,`author_id`,`description`,`created`,`last_updated`,`views`,`score` FROM `chonmage_image_info` WHERE image_id = '".$image_id."'");
                $kekka = $r1->fetchArray(SQLITE3_ASSOC);
                if($kekka === FALSE){
                    return FALSE;
                }
            }else{
                $r1 = $this->db_obj->query("SELECT `original_format`,`image_title`,`author_id`,`description`,`created`,`last_updated`,`views`,`score` FROM `chonmage_image_info` WHERE image_id = '".$image_id."'");
                $kekka = $r1->fetch_assoc();
                if($kekka === NULL){
                    return FALSE;
                }
            }
            
            $kekka["tags"] = array();
            
            if($this->config["use_sqlite"]){
                $r2 = $this->db_obj->query("SELECT `tag_name` FROM `chonmage_tags` WHERE image_id = '".$image_id."'");
                for($cnt = 0; $kekka_2 = $r2->fetchArray(SQLITE3_ASSOC); $cnt++){
                    $kekka["tags"][$cnt] = $kekka_2["tag_name"];
                }
            }else{
                $r2 = $this->db_obj->query("SELECT `tag_name` FROM `chonmage_tags` WHERE image_id = '".$image_id."'");
                for($cnt = 0; $kekka_2 = $r2->fetch_assoc(); $cnt++){
                    $kekka["tags"][$cnt] = $kekka_2["tag_name"];
                }
            }
            
            return $kekka;
        }else{
            $path = dirname(__FILE__)."/".$this->config["image_dir"]."/".$image_id.".ini";
            if(file_exists($path)){
                $kekka = parse_ini_file($path);
                
                if(strlen($kekka["tags"]) >= 1){
                    $tag = explode(" ",$kekka["tags"]);
                    $tag = array_unique($tag);
                    
                    $kekka["tags"] = $tag;
                }else{
                    $kekka["tags"] = NULL;
                }
                
                return $kekka;
            }else{
                return FALSE;
            }
        }
    }
    
    
    function update_image_info($image_id,$author_id = NULL,$title = NULL,$description = NULL,$tags = NULL){
        $image_id = $this->escape_image_id($image_id);
        
        $ima = date("Y-m-d H:i:s");
        
        $tags = $this->adjust_tags($tags);
        
        if($this->config["use_db"] <= 1){
            $base_dir = dirname(__FILE__)."/".$this->config["image_dir"]."/";
            
            $ini_data = parse_ini_file($base_dir.$image_id.".ini");
            if($ini_data === FALSE){
                return FALSE;
            }
            
            $ini_h = fopen($base_dir.$image_id.".ini","w");
            fwrite($ini_h,"image_id=\"".$image_id."\"\n");
            if($author_id !== NULL){
                fwrite($ini_h,"author_id=\"".addslashes($author_id)."\"\n");
            }else{
                fwrite($ini_h,"author_id=\"".addslashes($ini_data["author_id"])."\"\n");
            }
            fwrite($ini_h,"original_format=\"".addslashes($ini_data["original_format"])."\"\n");
            if($title !== NULL){
                fwrite($ini_h,"image_title=\"".addslashes($title)."\"\n");
            }else{
                fwrite($ini_h,"image_title=\"".addslashes($ini_data["image_title"])."\"\n");
            }
            if($description !== NULL){
                fwrite($ini_h,"description=\"".addslashes($description)."\"\n");
            }else{
                fwrite($ini_h,"description=\"".addslashes($ini_data["description"])."\"\n");
            }
            fwrite($ini_h,"created=\"".$ini_data["created"]."\"\n");
            fwrite($ini_h,"last_updated=\"".$ima."\"\n");
            fwrite($ini_h,"views=".$ini_data["views"]."\n");
            fwrite($ini_h,"score=".$ini_data["score"]."\n");
            if($tags !== NULL){
                fwrite($ini_h,"tags=\"".addslashes($tags)."\"\n");
            }else{
                fwrite($ini_h,"tags=\"".addslashes($ini_data["tags"])."\"\n");
            }
            fclose($ini_h);
        }
        
        if($this->config["use_db"] >= 1){
            if($this->config["use_sqlite"]){
                if($author_id !== NULL){
                    $q_author_id = "`author_id` = '".$this->db_obj->escapeString($author_id)."',";
                }else{
                    $q_author_id = "";
                }
                if($title !== NULL){
                    $q_title = "`image_title` = '".$this->db_obj->escapeString($title)."',";
                }else{
                    $q_title = "";
                }
            }else{
                if($author_id !== NULL){
                    $q_author_id = "`author_id` = '".$this->db_obj->real_escape_string($author_id)."',";
                }else{
                    $q_author_id = "";
                }
                if($title !== NULL){
                    $q_title = "`image_title` = '".$this->db_obj->real_escape_string($title)."',";
                }else{
                    $q_title = "";
                }
            }
            
            if($this->config["use_db"] == 2){
                if($this->config["use_sqlite"]){
                    if($description !== NULL){
                        $q_description = "`description` = '".$this->db_obj->escapeString($description)."',";
                    }else{
                        $q_description = "";
                    }
                    $r1 = $this->db_obj->query("UPDATE `chonmage_image_info` SET ".$q_author_id.$q_title.$q_description."`last_updated` = '".$ima."' WHERE `image_id` = '".$image_id."'");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                        return FALSE;
                    }
                }else{
                    if($description !== NULL){
                        $q_description = "`description` = '".$this->db_obj->real_escape_string($description)."',";
                    }else{
                        $q_description = "";
                    }
                    $r1 = $this->db_obj->query("UPDATE `chonmage_image_info` SET ".$q_author_id.$q_title.$q_description."`last_updated` = '".$ima."' WHERE `image_id` = '".$image_id."'");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                        return FALSE;
                    }
                }
            }else{
                if($this->config["use_sqlite"]){
                    $r1 = $this->db_obj->query("UPDATE `chonmage_image_info` SET ".$q_author_id.$q_title."`last_updated` = '".$ima."' WHERE `image_id` = '".$image_id."'");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                        return FALSE;
                    }
                }else{
                    $r1 = $this->db_obj->query("UPDATE `chonmage_image_info` SET ".$q_author_id.$q_title."`last_updated` = '".$ima."' WHERE `image_id` = '".$image_id."'");
                    if($r1 == FALSE){
                        print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                        return FALSE;
                    }
                }
            }
            
            if($tags !== NULL){
                if($this->config["use_sqlite"]){
                    $r2 = $this->db_obj->query("DELETE FROM `chonmage_tags` WHERE `image_id` = '".$image_id."'");
                    if($r2 == FALSE){
                        print "SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                        return FALSE;
                    }
                }else{
                    $r2 = $this->db_obj->query("DELETE FROM `chonmage_tags` WHERE `image_id` = '".$image_id."'");
                    if($r2 == FALSE){
                        print "SQLクエリ2の実行に失敗しました。".$this->db_obj->error;
                        return FALSE;
                    }
                }
                
                if(strlen($tags) >= 1){
                    $tag = explode(" ",$tags);
                    $tag = array_unique($tag);
                    $tag_kosu = count($tag);
                    
                    for($tag_c = 0; $tag_c < $tag_kosu; $tag_c++){
                        if($this->config["use_sqlite"]){
                            $r2 = $this->db_obj->query("INSERT INTO `chonmage_tags`(`image_id`,`tag_name`) VALUES ('".$image_id."','".$this->db_obj->escapeString($tag[$tag_c])."')");
                            if($r2 == FALSE){
                                print "SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                                return FALSE;
                            }
                        }else{
                            $r2 = $this->db_obj->query("INSERT INTO `chonmage_tags`(`image_id`,`tag_name`) VALUES ('".$image_id."','".$this->db_obj->real_escape_string($tag[$tag_c])."')");
                            if($r2 == FALSE){
                                print "SQLクエリ2の実行に失敗しました。".$this->db_obj->error;
                                return FALSE;
                            }
                        }
                    }
                }
            }
        }
        
        return TRUE;
    }
    
    
    function get_recent_images(){
        if($this->config["use_db"] <= 1){
            return explode("\n",str_replace("\r","",file_get_contents(dirname(__FILE__)."/".$this->config["image_dir"]."/recent_images.ini")));
        }else{
            $kekka = array();
            
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT `image_id` FROM `chonmage_image_info` ORDER BY `created` DESC LIMIT 50");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetchArray(SQLITE3_ASSOC); $cnt++){
                    $kekka[$cnt] = $data["image_id"];
                }
            }else{
                $r1 = $this->db_obj->query("SELECT `image_id` FROM `chonmage_image_info` ORDER BY `created` DESC LIMIT 50");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetch_assoc(); $cnt++){
                    $kekka[$cnt] = $data["image_id"];
                }
            }
            
            return $kekka;
        }
    }
    
    
    function search_image($q,$start,$rows,$key = "created",$asc = FALSE){
        if($this->config["use_db"] == 0){
            print "データベースを使用しないよう設定されています。検索は利用できません。";
            return FALSE;
        }else{
            if($asc){
                $junjyo = "ASC";
            }else{
                $junjyo = "DESC";
            }
            
            $query = "";
            
            if(!empty($q["tags"])){
                $tags = $this->adjust_tags($q["tags"]);
                
                if(strlen($tags) >= 1){
                    $tag = explode(" ",$tags);
                    $tag = array_unique($tag);
                    $tag_kosu = count($tag);
                    
                    $tag_q = "";
                    for($cnt = 0; $cnt < $tag_kosu; $cnt++){
                        if($cnt != 0){
                            $tag_q = $tag_q.",";
                        }
                        
                        if($this->config["use_sqlite"]){
                            $tag_q = $tag_q."'".$this->db_obj->escapeString($tag[$cnt])."'";
                        }else{
                            $tag_q = $tag_q."'".$this->db_obj->real_escape_string($tag[$cnt])."'";
                        }
                    }
                    
                    $query = "SELECT `chonmage_image_info`.* FROM `chonmage_image_info`, `chonmage_tags` WHERE ((`chonmage_image_info`.`image_id` = `chonmage_tags`.`image_id`) AND (`chonmage_tags`.`tag_name` IN (".$tag_q."))) GROUP BY `chonmage_tags`.`image_id` HAVING COUNT(`chonmage_tags`.`image_id`) = ".$tag_kosu;
                }
            }
            
            if(!empty($q["author_id"])){
                if(strlen($query) >= 1){
                    if($this->config["use_sqlite"]){
                        $query = $query." AND `chonmage_image_info`.`author_id` = '".$this->db_obj->escapeString($q["author_id"])."'";
                    }else{
                        $query = $query." AND `chonmage_image_info`.`author_id` = '".$this->db_obj->real_escape_string($q["author_id"])."'";
                    }
                }else{
                    if($this->config["use_sqlite"]){
                        $query = "SELECT `chonmage_image_info`.* FROM `chonmage_image_info` WHERE `chonmage_image_info`.`author_id` = '".$this->db_obj->escapeString($q["author_id"])."'";
                    }else{
                        $query = "SELECT `chonmage_image_info`.* FROM `chonmage_image_info` WHERE `chonmage_image_info`.`author_id` = '".$this->db_obj->real_escape_string($q["author_id"])."'";
                    }
                }
            }
            
            if(strlen($query) == 0){
                return FALSE;
            }
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query($query." ORDER BY `chonmage_image_info`.`".$this->db_obj->escapeString($key)."` ".$junjyo." LIMIT ".intval($start).",".intval($rows));
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                    
                $r2 = $this->db_obj->query("SELECT COUNT(*) AS `cnt` FROM (".$query.") AS `kekka`");
                if($r2 == FALSE){
                    print "SQLクエリ2の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                    
                $kekka = array();
                for($cnt = 0; $data = $r1->fetchArray(SQLITE3_ASSOC); $cnt++){
                    $kekka["image_id"][$cnt] = $data["image_id"];
                    $kekka["original_format"][$cnt] = $data["original_format"];
                    $kekka["image_title"][$cnt] = $data["image_title"];
                    $kekka["author_id"][$cnt] = $data["author_id"];
                    $kekka["created"][$cnt] = $data["created"];
                    $kekka["last_updated"][$cnt] = $data["last_updated"];
                    $kekka["views"][$cnt] = $data["views"];
                    $kekka["score"][$cnt] = $data["score"];
                }
                    
                $data = $r2->fetchArray(SQLITE3_ASSOC);
                if($data["cnt"] == 0){
                    return NULL;
                }else{
                    $kekka["count"] = $data["cnt"];
                    return $kekka;
                }
            }else{
                $r1 = $this->db_obj->query($query." ORDER BY `chonmage_image_info`.`".$this->db_obj->real_escape_string($key)."` ".$junjyo." LIMIT ".intval($start).",".intval($rows));
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                    
                $r2 = $this->db_obj->query("SELECT COUNT(*) AS `cnt` FROM (".$query.") AS `kekka`");
                if($r2 == FALSE){
                    print "SQLクエリ2の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                    
                $kekka = array();
                for($cnt = 0; $data = $r1->fetch_assoc(); $cnt++){
                    $kekka["image_id"][$cnt] = $data["image_id"];
                    $kekka["original_format"][$cnt] = $data["original_format"];
                    $kekka["image_title"][$cnt] = $data["image_title"];
                    $kekka["author_id"][$cnt] = $data["author_id"];
                    $kekka["created"][$cnt] = $data["created"];
                    $kekka["last_updated"][$cnt] = $data["last_updated"];
                    $kekka["views"][$cnt] = $data["views"];
                    $kekka["score"][$cnt] = $data["score"];
                }
                    
                $data = $r2->fetch_assoc();
                if($data["cnt"] == 0){
                    return NULL;
                }else{
                    $kekka["count"] = $data["cnt"];
                    return $kekka;
                }
            }
        }
    }
    
    
    function get_years(){
        if($this->config["use_db"] == 2){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT strftime('%Y',`created`) AS `years` FROM `chonmage_image_info` GROUP BY strftime('%Y',`created`)");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetchArray(SQLITE3_ASSOC); $cnt++){
                    $kekka[$cnt] = $data["years"];
                }
            }else{
                $r1 = $this->db_obj->query("SELECT date_format(`created`,'%Y') AS `years` FROM `chonmage_image_info` GROUP BY date_format(`created`,'%Y')");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetch_assoc(); $cnt++){
                    $kekka[$cnt] = $data["years"];
                }
            }
            
            return $kekka;
        }else{
            $path = glob(dirname(__FILE__)."/".$this->config["image_dir"]."/[0-9][0-9][0-9][0-9]");
            
            $kekka = array();
            $kosu = count($path);
            for($cnt = 0; $cnt < $kosu; $cnt++){
                $kekka[$cnt] = intval(basename($path[$cnt]));
            }
            
            return $kekka;
        }
    }
    
    
    function get_months($year){
        if($this->config["use_db"] == 2){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT strftime('%m',`created`) AS `months` FROM `chonmage_image_info` WHERE strftime('%Y',`created`) = '".intval($year)."' GROUP BY strftime('%m',`created`)");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetchArray(SQLITE3_ASSOC); $cnt++){
                    $kekka[$cnt] = intval($data["months"]);
                }
            }else{
                $r1 = $this->db_obj->query("SELECT date_format(`created`,'%c') AS `months` FROM `chonmage_image_info` WHERE date_format(`created`,'%Y') = '".intval($year)."' GROUP BY date_format(`created`,'%m')");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetch_assoc(); $cnt++){
                    $kekka[$cnt] = $data["months"];
                }
            }
            
            return $kekka;
        }else{
            $path = glob(dirname(__FILE__)."/".$this->config["image_dir"]."/".intval($year)."/[0-9][0-9]");
            
            $kekka = array();
            $kosu = count($path);
            for($cnt = 0; $cnt < $kosu; $cnt++){
                $kekka[$cnt] = abs(basename($path[$cnt]));
            }
            
            return $kekka;
        }
    }
    
    
    function get_days($year,$month){
        if($this->config["use_db"] == 2){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT strftime('%d',`created`) AS `days` FROM `chonmage_image_info` WHERE strftime('%Y%m',`created`) = '".intval($year).str_pad(mb_ereg_replace("[^0-9]","",$month),2,"0",STR_PAD_LEFT)."' GROUP BY strftime('%d',`created`)");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetchArray(SQLITE3_ASSOC); $cnt++){
                    $kekka[$cnt] = intval($data["days"]);
                }
            }else{
                $r1 = $this->db_obj->query("SELECT date_format(`created`,'%e') AS `days` FROM `chonmage_image_info` WHERE date_format(`created`,'%Y%m') = '".intval($year).str_pad(mb_ereg_replace("[^0-9]","",$month),2,"0",STR_PAD_LEFT)."' GROUP BY date_format(`created`,'%d')");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetch_assoc(); $cnt++){
                    $kekka[$cnt] = $data["days"];
                }
            }
            
            return $kekka;
        }else{
            $path = glob(dirname(__FILE__)."/".$this->config["image_dir"]."/".intval($year)."/".str_pad(mb_ereg_replace("[^0-9]","",$month),2,"0",STR_PAD_LEFT)."/[0-9][0-9]");
            
            $kekka = array();
            $kosu = count($path);
            for($cnt = 0; $cnt < $kosu; $cnt++){
                $kekka[$cnt] = abs(basename($path[$cnt]));
            }
            
            return $kekka;
        }
    }
    
    
    function get_image_list_of_day($year,$month,$day){
        if($this->config["use_db"] == 2){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT `image_id` FROM `chonmage_image_info` WHERE strftime('%Y%m%d',`created`) = '".intval($year).str_pad(mb_ereg_replace("[^0-9]","",$month),2,"0",STR_PAD_LEFT).str_pad(mb_ereg_replace("[^0-9]","",$day),2,"0",STR_PAD_LEFT)."'");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->lastErrorMsg();
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetchArray(SQLITE3_ASSOC); $cnt++){
                    $kekka[$cnt] = $data["image_id"];
                }
            }else{
                $r1 = $this->db_obj->query("SELECT `image_id` FROM `chonmage_image_info` WHERE date_format(`created`,'%Y%m%d') = '".intval($year).str_pad(mb_ereg_replace("[^0-9]","",$month),2,"0",STR_PAD_LEFT).str_pad(mb_ereg_replace("[^0-9]","",$day),2,"0",STR_PAD_LEFT)."'");
                if($r1 == FALSE){
                    print "SQLクエリ1の実行に失敗しました。".$this->db_obj->error;
                    return FALSE;
                }
                
                for($cnt = 0; $data = $r1->fetch_assoc(); $cnt++){
                    $kekka[$cnt] = $data["image_id"];
                }
            }
            return $kekka;
        }else{
            return array_values(array_filter(explode("\n",str_replace("\r","",file_get_contents(dirname(__FILE__)."/".$this->config["image_dir"]."/".intval($year)."/".str_pad(mb_ereg_replace("[^0-9]","",$month),2,"0",STR_PAD_LEFT)."/".str_pad(mb_ereg_replace("[^0-9]","",$day),2,"0",STR_PAD_LEFT)."/image_list.ini"))),"strlen"));
        }
    }
    
    
    function get_image_size_info($size_name = NULL){
        $size_name = str_replace($this->escape_list,"",$size_name);
        
        if($size_name !== NULL){
            $image_size_key = array_search($size_name,$this->image_size["size_name"]);
            
            if($image_size_key === FALSE){
                return FALSE;
            }else{
                $image_size_info = array();
                
                $image_size_info["size_name"] = $this->image_size["size_name"][$image_size_key];
                $image_size_info["width"] = $this->image_size["width"][$image_size_key];
                $image_size_info["height"] = $this->image_size["height"][$image_size_key];
                $image_size_info["background"] = $this->image_size["background"][$image_size_key];
                $image_size_info["file_format"] = $this->image_size["file_format"][$image_size_key];
                $image_size_info["expand"] = $this->image_size["expand"][$image_size_key];
                $image_size_info["scaling_method"] = $this->image_size["scaling_method"][$image_size_key];
                $image_size_info["compression_quality"] = $this->image_size["compression_quality"][$image_size_key];
                
                return $image_size_info;
            }
        }else{
            return $this->image_size;
        }
    }
    
    
    function original_exists($image_id){
        $image_id = $this->escape_image_id($image_id);
        
        if($this->config["use_db"] == 2){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT `original_format` FROM `chonmage_image_info`,`chonmage_image_blob` WHERE `chonmage_image_info`.`image_id` = '".$image_id."' AND `chonmage_image_blob`.`image_id` = '".$image_id."' AND `chonmage_image_blob`.`size_name` = 'original'");
                $kekka = $r1->fetchArray(SQLITE3_ASSOC);
                if($kekka === FALSE){
                    return FALSE;
                }
            }else{
                $r1 = $this->db_obj->query("SELECT `original_format` FROM `chonmage_image_info`,`chonmage_image_blob` WHERE `chonmage_image_info`.`image_id` = '".$image_id."' AND `chonmage_image_blob`.`image_id` = '".$image_id."' AND `chonmage_image_blob`.`size_name` = 'original'");
                $kekka = $r1->fetch_assoc();
                if($kekka === NULL){
                    return FALSE;
                }
            }
            
            return $kekka["original_format"];
        }else{
            $info = $this->get_image_info($image_id);
            
            if(file_exists(dirname(__FILE__)."/".$this->config["image_dir"]."/".$image_id.".".$info["original_format"])){
                return $info["original_format"];
            }else{
                return FALSE;
            }
        }
    }
    
    
    function get_blob($image_id,$size_name){
        $image_id = $this->escape_image_id($image_id);
        $size_name = str_replace($this->escape_list,"",$size_name);
        
        if($this->config["use_db"] == 2){
            if($this->config["use_sqlite"]){
                $r1 = $this->db_obj->query("SELECT `image_blob` FROM `chonmage_image_blob` WHERE image_id = '".$image_id."' AND size_name = '".$this->db_obj->escapeString($size_name)."'");
                $kekka = $r1->fetchArray(SQLITE3_ASSOC);
                if($kekka === FALSE){
                    return FALSE;
                }
            }else{
                $r1 = $this->db_obj->query("SELECT `image_blob` FROM `chonmage_image_blob` WHERE image_id = '".$image_id."' AND size_name = '".$this->db_obj->real_escape_string($size_name)."'");
                $kekka = $r1->fetch_assoc();
                if($kekka === NULL){
                    return FALSE;
                }
            }
            
            return $kekka["image_blob"];
        }else{
            if($size_name !== "original"){
                $info = $this->get_image_size_info($size_name);
                
                if($info !== FALSE && file_exists(dirname(__FILE__)."/".$this->config["image_dir"]."/".$image_id."_".$size_name.".".$info["file_format"])){
                    return file_get_contents(dirname(__FILE__)."/".$this->config["image_dir"]."/".$image_id."_".$size_name.".".$info["file_format"]);
                }else{
                    return FALSE;
                }
            }else{
                $format = $this->original_exists($image_id);
                
                if($format !== FALSE && file_exists(dirname(__FILE__)."/".$this->config["image_dir"]."/".$image_id.".".$format)){
                    return file_get_contents(dirname(__FILE__)."/".$this->config["image_dir"]."/".$image_id.".".$format);
                }else{
                    return FALSE;
                }
            }
        }
    }
    
    
    function output_image($image_id,$size_name){
        $size_name = str_replace($this->escape_list,"",$size_name);
        
        $blob = $this->get_blob($image_id,$size_name);
        
        if($blob === FALSE){
            header("HTTP/1.1 404 Not Found");
            return FALSE;
        }else{
            if($size_name === "original"){
                $format = $this->original_exists($image_id);
                header("Content-Type: image/".$format);
            }else{
                $info = $this->get_image_size_info($size_name);
                header("Content-Type: image/".$info["file_format"]);
            }
            
            print $blob;
            
            return TRUE;
        }
    }
}
